// -------------------------- MAP
var map = new ol.Map({
    target: 'map',
    layers: [
        new ol.layer.Tile({
        source: new ol.source.OSM()
        })
    ],
    view: new ol.View({
        center: ol.proj.fromLonLat([5.7640, 45.1899]),
        zoom: 17
    })
});
// Markers
var markers = new ol.layer.Vector({
    source: new ol.source.Vector(),
    style: function (feature) {
        return new ol.style.Style({
            image: new ol.style.Icon({
                anchor: [0.5, 1],
                src: './res/marker.png',
                scale: [0.3, 0.3]
            }),
            text: new ol.style.Text({
                text: feature.get('description'),
                font: "italic 12px sans-serif",
                offsetX: 10,
                textAlign: "left",
                fill: new ol.style.Fill({
                color: "#FFFFFF"
                }),

                stroke: new ol.style.Stroke({
                color: "#5E8D74",
                width: 3
                })
            })
        })
    }
});
map.addLayer(markers);

// Lines
var lines = new ol.layer.Vector({
    source: new ol.source.Vector(),
    style: new ol.style.Style({
        stroke: new ol.style.Stroke({
            color: '#ffcc33',
            width: 10
        })
    })
});
map.addLayer(lines);

// -------------------------- FUNCTIONS
function clearMarkers() {
    lines.getSource().clear();
    markers.getSource().clear();
}

function addMarker(name, lon, lat) {
    let marker = new ol.Feature(new ol.geom.Point(ol.proj.fromLonLat([lon, lat])));
    marker.set('description', name);
    markers.getSource().addFeature(marker);
}

function addLine(coords) {
    let geometry = new ol.geom.MultiLineString([coords]);
    geometry.transform('EPSG:4326', 'EPSG:3857');
    let feature = new ol.Feature({
        geometry: geometry,
        name: 'Path'
    });
    lines.getSource().addFeature(feature);
}

// -------------------------- API
async function fetchApi(path) {
    let response = await fetch("/api/" + path);

    if (response.ok) {
        return await response.json();
    } else {
        console.log("[API] " + path + " > HTTP-Error: " + response.status);
    }
    return null;
}

async function callDevices(path) {
    return await fetchApi("devices/" + path);
}

// -------------------------- PAGE

function tick() {
    fetchApi("points").then((points) => {
        clearMarkers();
        for (const [name, p] of Object.entries(points)) {
            let coords = [];
            for (entry of p.history) {
                if (coords.length == 0)
                    addMarker(name, entry.lon, entry.lat);
                coords.push([entry.lon, entry.lat]);
            }
            addLine(coords);
        }
        document.getElementById('status').style.color = 'green';
        document.getElementById('status').textContent = "Affichage de " + Object.entries(points).length + " relais !";
    }).catch((ex) => {
        document.getElementById('status').style.color = 'red';
        document.getElementById('status').textContent = "Erreur réseau...";
    });
}

setInterval(tick, 5000);
tick();

function onRing() {
    callDevices("ring").then((result) => {
        console.log("ring");
        console.log(result);
    });
}

function onShutdown() {
    callDevices("shutdown").then((result) => {
        console.log("shutdown");
        console.log(result);
    });
}
