var storage = require('../storage/storage.js');

var mqtt = require('mqtt')
var client = null

function broadcast(data) {
    if (!storage.broadcast.canSendPacketByIdentifier(data))
        return false;
    let payload = {
        "port": 99,
        "confirmed": false,
        "payload_raw": Buffer.from([data]).toString('base64')
    };
    console.log("[LORA] Broadcast: " + JSON.stringify(payload));
    //client.publish('iot5-orientation/devices/+/down', payload);
    storage.broadcast.savePacketByIdentifier(data);
    return true;
}

function callRaw(identifier, data) {
    let payload = {
        "port": 99,
        "confirmed": false,
        "payload_raw": Buffer.from([data]).toString('base64')
    };
    console.log("[LORA] Call " + identifier + ": " + JSON.stringify(payload));
    //client.publish('iot5-orientation/devices/' + identifier + '/down', payload);
    return true;
}

function handleMessage(identifier, buffer) {
    let header = buffer.readUInt8(0);
    let actualPacketBit = (header&0xc0)>>6;
    switch (actualPacketBit) {
        case 0:// Full
            let latFull = buffer.readFloatLE(1);
            let lonFull = buffer.readFloatLE(5);
            console.log("FULL: " + latFull + ";" + lonFull);
            storage.points.addPointFull(identifier, lonFull, latFull);
            callRaw(identifier, [0x254]);// Ok
            break;

        case 2:// Shift
            let latByte = buffer.readUInt8(1);
            let lonByte = buffer.readUInt8(2);

            let latSign = header&0x20;
            let lonSign = header&0x4;

            let lat = (header&0x18)<<5 | latByte;
            let lon = (header&0x3)<<8 | lonByte;

            if(latSign)
                lat *= -1;
            if(lonSign)
                lon *= -1;

            lat /= 10_000;
            lon /= 10_000;

            console.log("REL: " + lat + ";" + lon);
            storage.points.addPointRelative(identifier, lon, lat);
            break;

        default:
            console.log("UNK");
            console.log(buffer);
            break;
    }
}

/*
let absolute = [0x00,0xff,0x9d,0x30,0x42,0x09,0x92,0x9a,0x40]; //44.1542931, 4.83032656
handleMessage("yolo", Buffer.from(absolute));

let shift = [0x84, 0x04, 0xc7]; //shift of 4,-199 should give 44.1546931,4.81032656
handleMessage("yolo", Buffer.from(shift));
*/

module.exports.start = function () {
    console.log("[LORA] Connecting...");
    client = mqtt.connect('mqtt://eu1.cloud.thethings.network', {
        clientId: `iot5_back_${Math.random().toString(16).slice(3)}`,
        clean: true,
        connectTimeout: 4000,
        username: 'iot5-orientation@ttn',
        password: 'NNSXS.M426ZAM6F72SGAMNNFR6DV5SZL4LUDCQCVJ43MI.BTYXLQP4JLUTD5XU4UBIEEWE7ZJYCJEVH3S6IFVEGYDZXUAOZ4XA',
        reconnectPeriod: 1000
    })

    client.on('connect', function (packet) {
        console.log("[LORA] Connect status: " + JSON.stringify(packet))
        console.log('[LORA] Events are ' + client.eventNames())
        client.subscribe('v3/iot5-orientation@ttn/devices/eui-e24f43fffe44d211/up', (err,granted) => {
            if(err) {
                console.log("[LORA] Subscribe error: " + JSON.stringify(err))
            }
    
            console.log("[LORA] Subscribed to devices: " + JSON.stringify(granted))
        })

        client.on('message', function (topic, payload, message) {
            console.log("[LORA] Message: " + topic + ":" + payload)
            let data = JSON.parse(payload);
            console.log("[LORA] " + data.end_device_ids.device_id + ": " + data.uplink_message.decoded_payload.bytes);
            handleMessage(data.end_device_ids.device_id, Buffer.from(data.uplink_message.decoded_payload.bytes));
        })
    })
}

module.exports.broadcastRing = function () {
    return broadcast(255);
}

module.exports.broadcastShutdown = function () {
    return broadcast(253);
}
